/*
 * PIC12F1840 8-PIN DIL
 * 975HZ PWM 
 * ANALOG SIGNAL IN A0 ON PIN 7
 * PWM OUT ON PIN 5
 * 
 * File:   newmain.c
 * Author: tpgof
 *
 * Created on 13 June 2024, 11:54
 */


// PIC12F1840 Configuration Bit Settings

// 'C' source line config statements

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = OFF      // Power-up Timer Enable (PWRT disabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)
#pragma config IESO = OFF       // Internal/External Switchover Mode (Internal/External Switchover Mode is disabled)
#pragma config FCMEN = OFF      // Fail-Safe Clock Monitor Enable (Fail-Safe Clock Monitor is disabled)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config PLLEN = OFF      // PLL Enable Bit (4x PLL disabled)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

// #pragma config statements should precede project file includes.
// Use project enums instead of #define for ON and OFF.

#include <xc.h>

// Define the clock frequency for delay calculations
#define _XTAL_FREQ 160000000  // 16 MHz internal oscillator
#define ADC_SAMPLES 16

// Function Prototypes
void initADC(void);
void initPWM(void);
unsigned int readADC(void);
void setPWMDutyCycle(unsigned int duty);

void main(void) {
    // Initialize the ADC and PWM modules
    initADC();
    initPWM();
    OSCCON = 0X78;

    while (1) {
        // Read the ADC value
        unsigned int adcValue = readADC();
        
        // Set the PWM duty cycle based on the ADC value
        setPWMDutyCycle(adcValue);
        
        // Small delay to stabilize readings
        __delay_ms(10);
    }
}

// Function to initialize the ADC
void initADC(void) {
    ADCON0 = 0x01;          // ADCON0: ADON=1 (Enable ADC), Channel select bits=000 (AN0)
    ADCON1 = 0x80;          // ADCON1: ADCS = Fosc/8 (Conversion clock select bits), right justify result
    ANSELAbits.ANSA0 = 1;   // ANSEL: AN0 as analog input
    TRISAbits.TRISA0 = 1;   // Set RA0/AN0 as input
}

// Function to initialize the PWM (CCP1)
void initPWM(void) {
    TRISAbits.TRISA2 = 0;   // Set RA5 as output (CCP1 pin)
    PR2 = 0xFF;             // Set PWM frequency to a higher value (increase frequency) was3f
    CCPR1L = 0x00;          // Initialize duty cycle to 0
    CCP1CON = 0x0C;         // CCP1CON: CCP1M3=1, CCP1M2=1 (PWM mode)
    T2CON = 0x06;           // T2CON: TMR2ON=1 (Timer2 ON), T2CKPS1=0, T2CKPS0=1 (Prescaler is 1)
}

// Function to read the ADC value from AN0 using raw values (swap with function below if you want to average))
unsigned int readADC(void) {
    __delay_us(5);           // Acquisition time
    ADCON0bits.GO_nDONE = 1;  // Start conversion
    while (ADCON0bits.GO_nDONE);  // Wait for conversion to complete
    return (unsigned int)((1023-((ADRESH << 8) + ADRESL))); // Return the 10-bit ADC result
}

// Function to read the ADC value from AN0 and average it over 16 values for smoothing
//unsigned int readADC(void) { // 
    //unsigned long sum = 0;
    //for (int i = 0; i < ADC_SAMPLES; i++) {
        //__delay_us(5);           // Acquisition time
       //ADCON0bits.GO_nDONE = 1;  // Start conversion
        //while (ADCON0bits.GO_nDONE);  // Wait for conversion to complete
        //sum += ((ADRESH << 8) + ADRESL); // Accumulate the ADC result
    //}
    //return (unsigned int)(1023-(sum / ADC_SAMPLES)); // Return the averaged result and reverse it
//}



// Function to set the PWM duty cycle
void setPWMDutyCycle(unsigned int duty) {
    unsigned int pwmValue = (duty >> 2) & 0xFF;  // Scale down to 8 bits for CCPR1L
    CCPR1L = pwmValue;                           // Set the PWM duty cycle
    CCP1CONbits.DC1B = duty & 0x03;              // Set the 2 LSBs of the PWM duty cycle
}